import { Model } from "../../model";
export class Transform extends Model {
    constructor(attrs) {
        super(attrs);
    }
}
Transform.__name__ = "Transform";
//# sourceMappingURL=transform.js.map